@@ -63,7 +63,7 @@ export BASEDIR=`pwd`
 # export CDNAME=debian
 
 # Building $codename cd set ...
-export CODENAME=buster
+export CODENAME=bullseye
 
 # By default use Debian installer packages from $CODENAME
 if [ -z "$DI_CODENAME" ]; then
@@ -87,7 +87,7 @@ fi
 #export DI_WWW_HOME=default
 
 # Version number, "2.2 r0", "2.2 r1" etc.
-export DEBVERSION="10.0.0"
+export DEBVERSION="11.0.0"
 
 # Official or non-official set.
 # NOTE: THE "OFFICIAL" DESIGNATION IS ONLY ALLOWED FOR IMAGES AVAILABLE
@@ -206,6 +206,11 @@ export JIGDO_CHECKSUM="md5"
 # The path to the keyring file relative to $TDIR/archive-keyring/
 #export ARCHIVE_KEYRING_FILE=usr/share/keyrings/debian-archive-keyring.gpg
 
+# Extra keys that you might want apt to trust. List their fingerprints
+# here and debian-cd will grab them from the user's keyring as needed
+# (The example here is the buster release key)
+#export ARCHIVE_EXTRA_KEYS="80D15823B7FD1561F9F7BCDDDC30D7C23CBBABEE"
+
 # By default we use debootstrap --no-check-gpg to find out the minimal set
 # of packages because there's no reason to not trust the local mirror. But
 # you can be paranoid and then you need to indicate the keyring to use to
@@ -434,6 +439,11 @@ if [ -d "/etc/ssl/ca-debian" ]; then
 fi
 export WGET="wget $WGET_OPTS"
 
+# Run the make_image step in parallel? Specify the number of calls to
+# use in parallel here if desired. Don't go too high - this *will*
+# thrash your IO!
+#export PARALLEL_MAKE_IMAGE=4
+
 # Set this to force the Release file(s) to say "stable". Used in first
 # Etch builds to allow us to build before the archive updated
 #EARLY_BUILD_HACK=1
